@@ -1,4 +1,4 @@
-(** Annotations. THIS MODULE IS UNFINISHED. *)
+(** Annotations *)
 
 (** Border styles *)
 type style = NoStyle | Solid | Dashed | Beveled | Inset | UnderlineStyle
@@ -28,7 +28,7 @@ type subtype =
   | Stamp
   | Caret
   | Ink
-  | Popup of annotation
+  | Popup of t
   | FileAttachment
   | Sound
   | Movie
@@ -41,14 +41,16 @@ type subtype =
   | Unknown
 
 (** Annotations. *)
-and annotation =
+and t =
   {subtype : subtype;
-   contents : string option;
+   annot_contents : string option;
+   subject : string option;
    rectangle : float * float * float * float;
    border : border;
-   rest : Pdf.pdfobject}
+   colour : (int * int * int) option;
+   annotrest : Pdf.pdfobject}
 
-val annotations_of_page : Pdf.pdfdoc -> Pdfdoc.page -> annotation list
+val annotations_of_page : Pdf.t -> Pdfpage.t -> t list
 (** Return the annotations on a page in a document. *)
 
 (*i val add_annotation : Pdf.pdfdoc -> Pdfdoc.page -> annotation -> unit i*)
