@@ -1,10 +1,9 @@
-(* \chaptertitle{PDFFun}{PDF Functions} *)
-open Utility
-open Pdf
+open Pdfutil
+open Pdfio
 
-(* \section{Types} *)
+(* Types *)
 
-(* \intf Postscript calculator functions. *)
+(* Postscript calculator functions. *)
 type calculator =
   | If of calculator list
   | IfElse of calculator list * calculator list
@@ -17,7 +16,7 @@ type calculator =
   | And | Bitshift | Eq | Ge | Gt | Le | Lt | Ne | Not
   | Or | Xor | Copy | Exch | Pop | Dup | Index | Roll
 
-(* \intf Sampled functions. *)
+(* Sampled functions. *)
 type sampled =
   {size : int list; 
    order : int;
@@ -26,32 +25,32 @@ type sampled =
    bps : int;
    samples : int32 array}
 
-(* \intf Interpolation functions. *)
+(* Interpolation functions. *)
 and interpolation =
   {c0 : float list;
    c1 : float list;
    n : float}
 
-(* \intf Stitching functions. *)
+(* Stitching functions. *)
 and stitching =
-  {functions : pdf_fun list;
+  {functions : t list;
    bounds : float list;
    stitch_encode : float list}
 
-(* \intf Collect the above types into a single type. *)
+(* Collect the above types into a single type. *)
 and pdf_fun_kind =
   | Interpolation of interpolation
   | Stitching of stitching
   | Sampled of sampled
   | Calculator of calculator list
 
-(* \intf Main type. *)
-and pdf_fun =
+(* Main type. *)
+and t =
   {func : pdf_fun_kind;
    domain : float list;
    range : float list option}
 
-(* \section{Printing functions} *)
+(* Printing functions *)
 
 (* Build a string of a calculator function. For debug only, since could exceed
 string length limit. *)
@@ -122,7 +121,7 @@ let rec print_function f =
       print_string "Calculator:\n";
       print_string (string_of_calculator c)
 
-(* \section{Parsing Calculator Functions} *)
+(* Parsing Calculator Functions *)
 let keyword_of_string = function
   | "abs" -> Abs | "add" -> Add | "atan" -> Atan | "ceiling" -> Ceiling
   | "cos" -> Cos | "cvr" -> Cvr | "div" -> Div | "exp" -> Exp
@@ -135,32 +134,38 @@ let keyword_of_string = function
   | "copy" -> Copy | "exch" -> Exch | "pop" -> Pop
   | "dup" -> Dup | "index" -> Index | "roll" -> Roll
   | s ->
-      flprint ("Bad keyword " ^ s); raise (Assert_failure ("", 0, 0))
+      Printf.eprintf "%s" ("Bad keyword " ^ s); assert false
 
 let parse_calculator s =
   let lexemes =
-    Cgenlex.lex (Pdfio.input_of_bytestream (bytestream_of_string s))
+    Pdfgenlex.lex (Pdfio.input_of_string s)
   in
     let rec strip_outer_braces = function
-      | Cgenlex.Ident ("{" | "}")::t ->
+      | Pdfgenlex.LexName ("{" | "}")::t ->
           rev (strip_outer_braces (rev t))
       | x -> x
     and group_operators = function
       | [] -> []
-      | Cgenlex.Ident "{"::t ->
-          let ops, rest = cleavewhile (neq (Cgenlex.Ident "}")) t in
+      | Pdfgenlex.LexName "{"::t ->
+          let ops, rest = cleavewhile (neq (Pdfgenlex.LexName "}")) t in
             ops::group_operators (tl rest)
       | h::t -> [h]::group_operators t
     and parse = function
       | [] -> []
-      | l::l'::[Cgenlex.Ident "ifelse"]::t ->
+      | l::l'::[Pdfgenlex.LexName "ifelse"]::t ->
           IfElse (procss l, procss l')::parse t
-      | l::[Cgenlex.Ident "if"]::t -> If (procss l)::parse t
-      | [Cgenlex.Ident "true"]::t -> Bool true::parse t
-      | [Cgenlex.Ident "false"]::t -> Bool false::parse t
-      | [Cgenlex.Float f]::t -> Float f::parse t
-      | [Cgenlex.Int i]::t-> Int (i32ofi i)::parse t (* FIXME: range *)
-      | [Cgenlex.Ident x]::t -> keyword_of_string x::parse t
+      | l::[Pdfgenlex.LexName "if"]::t -> If (procss l)::parse t
+      | [Pdfgenlex.LexName "true"]::t -> Bool true::parse t
+      | [Pdfgenlex.LexName "false"]::t -> Bool false::parse t
+      | [Pdfgenlex.LexReal f]::t ->
+          (* If the float is actually a whole number, convert to an Int. This
+          will be accepted as a float (where any number is ok) and as an int
+          (where an integer is required. Sometimes, buggy code has things
+          like 1.000000 where 1 is required. *)
+          (*(if f = floor f then Int (Int32.of_float f) else *)
+          Float f::parse t
+      | [Pdfgenlex.LexInt i]::t-> Int (i32ofi i)::parse t (* FIXME: range *)
+      | [Pdfgenlex.LexName x]::t -> keyword_of_string x::parse t
       | h::_ -> (*i Printf.eprintf (Cgenlex.string_of_tokens h); i*) raise (Failure "Bad lexeme")
     and procss lexemes =
       try
@@ -170,68 +175,68 @@ let parse_calculator s =
     in
       procss lexemes
 
-(* \section{Parsing functions} *)
+(* Parsing functions *)
 let rec parse_function pdf f =
-  let f = direct pdf f in
-    let getnum_direct o = getnum (direct pdf o) in
+  let f = Pdf.direct pdf f in
+    let getnum_direct o = Pdf.getnum (Pdf.direct pdf o) in
       let domain =
-        match lookup_fail "No /Domain" pdf "/Domain" f with
-        | Array ns -> map getnum_direct ns
-        | _ -> raise (PDFError "Bad /Domain")
-      and range =
-        match lookup_direct pdf "/Range" f with
-        | Some (Array ns) -> Some (map getnum_direct ns)
+        match Pdf.lookup_fail "No /Domain" pdf "/Domain" f with
+        | Pdf.Array ns -> map getnum_direct ns
+        | _ -> raise (Pdf.PDFError "Bad /Domain")
+      in let range =
+        match Pdf.lookup_direct pdf "/Range" f with
+        | Some (Pdf.Array ns) -> Some (map getnum_direct ns)
         | _ -> None
       in
       let func =
-        match lookup_fail "no /FunctionType" pdf "/FunctionType" f with
-        | Integer 0 ->
+        match Pdf.lookup_fail "no /FunctionType" pdf "/FunctionType" f with
+        | Pdf.Integer 0 ->
             let size =
-              match lookup_fail "no /Size (sampled fun)" pdf "/Size" f with
-              | Array ns ->
+              match Pdf.lookup_fail "no /Size (sampled fun)" pdf "/Size" f with
+              | Pdf.Array ns ->
                   map
                     (function
-                     | Integer n -> n
-                     | _ -> raise (PDFError "bad /Size (sampled fun)"))
+                     | Pdf.Integer n -> n
+                     | _ -> raise (Pdf.PDFError "bad /Size (sampled fun)"))
                     ns
-              | _ -> raise (PDFError "Bad /Size (sampled fun)")
+              | _ -> raise (Pdf.PDFError "Bad /Size (sampled fun)")
             in
               let order =
-                match lookup_direct pdf "/Order" f with
-                | Some (Integer n) -> n
+                match Pdf.lookup_direct pdf "/Order" f with
+                | Some (Pdf.Integer n) -> n
                 | _ -> 1
-              and encode =
-                match lookup_direct pdf "/Encode" f with
-                | Some (Array ns) when length ns = 2 * length size ->
-                    map getnum ns
+              in let encode =
+                match Pdf.lookup_direct pdf "/Encode" f with
+                | Some (Pdf.Array ns) when length ns = 2 * length size ->
+                    map Pdf.getnum ns
                 | _ ->
                     interleave_lists
                       (many 0. (length size))
                       (map (fun x -> float (x - 1)) size)
-              and decode =
-                match lookup_direct pdf "/Decode" f with
-                | Some (Array ns) -> map getnum ns
+              in let decode =
+                match Pdf.lookup_direct pdf "/Decode" f with
+                | Some (Pdf.Array ns) -> map Pdf.getnum ns
                 | _ ->
                     match range with
                     | Some r -> r
-                    | None -> raise (PDFError "No /Range")
+                    | None -> raise (Pdf.PDFError "No /Range")
               in
                 let bitspersample =
                   match
-                    lookup_fail "no /BitsPerSample" pdf "/BitsPerSample" f
+                    Pdf.lookup_fail "no /BitsPerSample" pdf "/BitsPerSample" f
                   with
-                  | Integer i -> i
-                  | _ -> raise (PDFError "Bad /BitsPerSample")
+                  | Pdf.Integer i -> i
+                  | _ -> raise (Pdf.PDFError "Bad /BitsPerSample")
                 in
                   let data =
                     Pdfcodec.decode_pdfstream pdf f; 
                     let samples =
                       fold_left ( * ) 1 size * (length decode / 2)
-                    and bitstream =
+                    in let bitstream =
                       match f with
-                      | Stream {contents = _, Got data} ->
-                          Pdfio.bitstream_of_input (Pdfio.input_of_bytestream data)
-                      | _ -> raise (Assert_failure ("", 0, 0))
+                      | Pdf.Stream {contents = _, Pdf.Got data} ->
+                          Pdfio.bitbytes_of_input (Pdfio.input_of_bytes data)
+                      | _ -> assert false
 
                     in
                       let data = Array.make samples 1l in
@@ -247,52 +252,52 @@ let rec parse_function pdf f =
                       decode = decode;
                       bps = bitspersample;
                       samples = data}
-        | Integer 2 ->
+        | Pdf.Integer 2 ->
             let c0 =
-              match lookup_direct pdf "/C0" f with
-              | Some (Array ns) -> map getnum_direct ns
+              match Pdf.lookup_direct pdf "/C0" f with
+              | Some (Pdf.Array ns) -> map getnum_direct ns
               | _ -> [0.]
-            and c1 =
-              match lookup_direct pdf "/C1" f with
-              | Some (Array ns) -> map getnum_direct ns
+            in let c1 =
+              match Pdf.lookup_direct pdf "/C1" f with
+              | Some (Pdf.Array ns) -> map getnum_direct ns
               | _ -> [1.]
-            and n =
-              getnum (lookup_fail "No /N in Type 2 fun" pdf "/N" f)
+            in let n =
+              Pdf.getnum (Pdf.lookup_fail "No /N in Type 2 fun" pdf "/N" f)
             in
               Interpolation {c0 = c0; c1 = c1; n = n}
-        | Integer 3 ->
+        | Pdf.Integer 3 ->
             let functions =
-              match lookup_fail "no /Functions" pdf "/Functions" f with
-              | Array fs -> fs
-              | _ -> raise (PDFError "Bad /Functions")
-            and bounds =
-              match lookup_fail "no /Bounds" pdf "/Bounds" f with
-              | Array fs -> fs
-              | _ -> raise (PDFError "Bad /Bounds")
-            and encode =
-              match lookup_fail "no /Encode" pdf "/Encode" f with
-              | Array fs -> fs
-              | _ -> raise (PDFError "Bad /Bounds")
+              match Pdf.lookup_fail "no /Functions" pdf "/Functions" f with
+              | Pdf.Array fs -> fs
+              | _ -> raise (Pdf.PDFError "Bad /Functions")
+            in let bounds =
+              match Pdf.lookup_fail "no /Bounds" pdf "/Bounds" f with
+              | Pdf.Array fs -> fs
+              | _ -> raise (Pdf.PDFError "Bad /Bounds")
+            in let encode =
+              match Pdf.lookup_fail "no /Encode" pdf "/Encode" f with
+              | Pdf.Array fs -> fs
+              | _ -> raise (Pdf.PDFError "Bad /Bounds")
             in
               Stitching
                 {functions = map (parse_function pdf) functions;
                  bounds = map getnum_direct bounds;
                  stitch_encode = map getnum_direct encode}
-        | Integer 4 ->
+        | Pdf.Integer 4 ->
             (* Read contents of stream, build string, parse. *)
             Pdfcodec.decode_pdfstream pdf f;
             begin match f with
-            | Stream {contents = _, Got data} ->
-                Calculator (parse_calculator (string_of_bytestream data))
-            | _ -> raise (PDFError "This is not a function")
+            | Pdf.Stream {contents = _, Pdf.Got data} ->
+                Calculator (parse_calculator (string_of_bytes data))
+            | _ -> raise (Pdf.PDFError "This is not a function")
             end
-        | _ -> raise (PDFError "Unknown function type")
+        | _ -> raise (Pdf.PDFError "Unknown function type")
       in
         {domain = domain; range = range; func = func}
 
-(* \section{Evaluating Sampled Functions} *)
+(* Evaluating Sampled Functions *)
 
-(* \intf Inappropriate inputs have been given to a function. *)
+(* Inappropriate inputs have been given to a function. *)
 exception BadFunctionEvaluation of string
 
 let interpolate x xmin xmax ymin ymax =
@@ -303,15 +308,17 @@ only sensibly for one-dimensional functions. Although results will be
 produced for higher dimensions, the results will not be fully accurate. *)
 let eval_function_sampled f s clamped_inputs =
   (* 1. Encode the input values *)
+  (*i Printf.printf "eval_function_sampled: length of clamped_inputs %i input is
+   * %f\n" (length clamped_inputs) (hd clamped_inputs); i*)
   let range =
     match f.range with
     | None -> raise (BadFunctionEvaluation "No Range")
     | Some r -> r
   in
     let d, d' = split (pairs_of_list f.domain)
-    and e, e' = split (pairs_of_list s.encode)
-    and dec, dec' = split (pairs_of_list s.decode)
-    and r, r' = split (pairs_of_list range) in
+    in let e, e' = split (pairs_of_list s.encode)
+    in let dec, dec' = split (pairs_of_list s.decode)
+    in let r, r' = split (pairs_of_list range) in
       let encoded_inputs =
         map5 interpolate clamped_inputs d d' e e'
       in
@@ -329,11 +336,14 @@ let eval_function_sampled f s clamped_inputs =
               raise (BadFunctionEvaluation "Incompatible /Size with inputs");
             let pos =
               let multipliers =
+                let inputs' = if length inputs = 1 then [] else ilist 1 (length inputs - 1) in
                 1::
                 map
-                  (function x -> fold_left ( * ) 1 (take size x))
-                  (ilist_fail_null 1 (length inputs - 1))
+                  (function x -> fold_left ( * ) 1 (take size x)) inputs'
+                  (* (ilist_fail_null 1 (length inputs - 1)) *)
               in
+                (*i Printf.printf "%i inputs and %i multipliers" (length inputs)
+                (length multipliers); i*)
                 fold_left ( + ) 0 (map2 ( * ) inputs multipliers) * vals_to_read
             in
               Array.to_list (Array.sub s.samples pos vals_to_read)
@@ -341,12 +351,14 @@ let eval_function_sampled f s clamped_inputs =
           (* 3. Read values from table. For now, just linear iterpolation. *)
           let ceilings =
             map (fun x -> toint (ceil x)) clamped_encoded_inputs
-          and floors =
+          in let floors =
             map (fun x -> toint (floor x)) clamped_encoded_inputs
           in
             let outputs = 
               let ceiling_results = read_table ceilings
-              and floor_results = read_table floors in
+              in let floor_results = read_table floors in
+                (*i Printf.printf "length of floors: %i, length of ceilings %i\n"
+                (List.length ceiling_results) (List.length floor_results); i*)
                 map2
                   (fun x y ->
                      Int32.to_float x /. 2. +. Int32.to_float y /. 2.)
@@ -364,11 +376,11 @@ let eval_function_sampled f s clamped_inputs =
               in
                 map3 (fun x r r' -> fmin (fmax x r) r') outputs_decoded r r'
 
-(* \section{Evaluating Calculator Functions} *)
+(* Evaluating Calculator Functions *)
 let eval_function_calculator clamped_inputs ops =
   let s =
     ref (map (fun i -> Float i) (rev clamped_inputs))
-  and typecheck () =
+  in let typecheck () =
     raise (BadFunctionEvaluation "Type error")
   in
     let rec getfloat () =
@@ -386,8 +398,8 @@ let eval_function_calculator clamped_inputs ops =
       let x = getint () in x, getint ()
     in
       let rec eval k =
-      (*i flprint (string_of_calculator !s);
-      Printf.printf "Now op %s\n\n" (string_of_calculator [k]); i*)
+      (* flprint (string_of_calculator !s);
+      Printf.printf "Now op %s\n\n" (string_of_calculator [k]); *)
       match k with
       | If l ->
           begin match !s with
@@ -486,7 +498,7 @@ let eval_function_calculator clamped_inputs ops =
           end
       | Neg ->
           begin match !s with
-          | Float f::r -> s := Float (~-.f)::r
+          | Float f::r -> s := Float (-.f)::r
           | Int i::r ->
               let out =
                 if i = Int32.min_int
@@ -639,12 +651,12 @@ let eval_function_calculator clamped_inputs ops =
           in
             begin match !s with
             | Int j::Int n::r when n >= 1l ->
-                let j = i32toi j and n = i32toi n in
+                let j = i32toi j in let n = i32toi n in
                   let vals, rest = cleave r n in
                     let newvals =
                       if j > 0
                         then rotate j vals
-                        else rotate_o ~-j vals
+                        else rotate_o (-j) vals
                     in
                     s := newvals @ rest
             | _ -> typecheck ()
@@ -655,14 +667,15 @@ let eval_function_calculator clamped_inputs ops =
       rev_map
         (function
          | Float x -> x
-         | _ -> raise (BadFunctionEvaluation "Type 4"))
+         | Int i -> Int32.to_float i (* Added as a FIX, but we should look at this again...*)
+         | _ -> raise (BadFunctionEvaluation "Type 4-a"))
         !s
     with
-      _ -> raise (BadFunctionEvaluation "Type 4")
+      e -> flprint (Printexc.to_string e); raise (BadFunctionEvaluation "Type 4-b")
 
-(* \section{Evaluating functions} *)
+(* Evaluating functions *)
 
-(* \intf Evaluate a function on some inputs. *)
+(* Evaluate a function on some inputs. *)
 let rec eval_function f inputs =
   let rec clampvals vals domain =
     let clampval v d d' =
@@ -692,9 +705,17 @@ let rec eval_function f inputs =
             in
               flatten (map (interp f.n f.c0 f.c1) clamped_inputs)
         | Stitching s ->
+            (* iPrintf.printf "First input to function: %f\n" (hd
+             * clamped_inputs); *)
             match clamped_inputs, f.domain with
             | [i], [d0; d1] ->
                 let points = [d0] @ s.bounds @ [d1] in
+                  (*i flprint "points: ";
+                  iter (Printf.printf "%f ") points;
+                  flprint "\n";
+                  flprint "s.stitch_encode: ";
+                  iter (Printf.printf "%f ") s.stitch_encode;
+                  flprint "\n"; i*)
                   let rec extract_subfunction points funs n =
                     match points, funs with
                     | p::p'::_, f::_ when i >= p && i < p' -> p, p', f, n
@@ -704,12 +725,16 @@ let rec eval_function f inputs =
                         raise (BadFunctionEvaluation "stitching: funs")
                   in
                     let d0', d1', f, n = extract_subfunction points s.functions 0 in
+                     (*i Printf.printf "d0 = %f, d1 = %f, n = %i\n" d0' d1' n;
+                      * i*)
                      let encode =
                        try
                          let a, b, c, d =
                            select (n + 1) points, select (n + 2) points,
-                           select (n + 1) s.stitch_encode, select (n + 2) s.stitch_encode
+                           select (n * 2 + 1) s.stitch_encode, select (n * 2 + 2) s.stitch_encode
                          in
+                           (* Printf.printf "a = %f, b %f, c = %f, d = %f\n" a b
+                            * c d; *)
                            fun x -> interpolate x a b c d
                        with
                          Invalid_argument "select" ->
@@ -718,9 +743,14 @@ let rec eval_function f inputs =
                        eval_function f [encode i]
             | _ -> raise (BadFunctionEvaluation "stitching: Bad arity")
       in
+        let outputs = 
         match f.range with
         | None -> outputs
         | Some range -> clampvals outputs range
+        in
+          (* iter (Printf.printf "%f ") outputs;
+          flprint "\n"; *)
+          outputs
 
 (*i let eval_function f vals =
   Printf.printf "eval_function\n";
@@ -766,7 +796,7 @@ and extra_entries_of_function pdf f =
 and pdfobject_of_function pdf f =
   let domain =
     Pdf.Array (map (function x -> Pdf.Real x) f.domain)
-  and range =
+  in let range =
     match f.range with
     | None -> []
     | Some fs -> ["/Range", Pdf.Array (map (function x -> Pdf.Real x) fs)]
