@@ -1,10 +1,10 @@
-(** Parsing Fonts and Extracting Text *)
+(** Parsing fonts and extracting text from content streams and PDF strings *)
 
 (** {2 Data Types } *)
 
 type type3_glpyhs =
   {fontbbox : float * float * float * float;
-   fontmatrix : Transform.transform_matrix;
+   fontmatrix : Pdftransform.transform_matrix;
    charprocs : (string * Pdf.pdfobject) list;
    type3_resources : Pdf.pdfobject}
 
@@ -63,8 +63,6 @@ type standard_font =
   | Symbol
   | ZapfDingbats
 
-val string_of_standard_font : standard_font -> string
-
 type cid_system_info =
   {registry : string;
    ordering : string;
@@ -86,35 +84,73 @@ type font =
   | SimpleFont of simple_font
   | CIDKeyedFont of string * composite_CIDfont * cmap_encoding
 
+(** {2 String representations of fonts } *)
+
+(** Returns a string such as "Times-Bold" for Pdftext.TimesBold etc. *)
+val string_of_standard_font : standard_font -> string
+
+(** Parses a string such as "/Times-Bold" or "/TimesNewRoman,Bold" to Pdftext.TimesRomanBold etc. *)
+val standard_font_of_name : string -> standard_font option
+
+(** A debug string for the whole font datatype. *)
+val string_of_font : font -> string
+
 (** {2 Reading a Font} *)
 
 (** Read a font from a given document and object *)
-val read_font : Pdf.pdfdoc -> Pdf.pdfobject -> font
+val read_font : Pdf.t -> Pdf.pdfobject -> font
 
-(** {2 Text Extraction} *)
+(** {2 Writing a Font} *)
+
+(** Write a font to a given document, returning the object number for the main
+font dictionary *)
+val write_font : Pdf.t -> font -> int
+
+(** {2 Utility functions} *)
+
+(** A list of unicode codepoints for a UTF8 string *)
+val codepoints_of_utf8 : string -> int list
+
+(** A UTF8 string for a list of unicode codepoints *)
+val utf8_of_codepoints : int list -> string
+
+(** {2 Text from strings outside page content} *)
+
+(** Take a pdf string (which will be either pdfdocencoding or UTF16BE) and
+return a string representing the same unicode codepoints in UTF8 *)
+val utf8_of_pdfdocstring : string -> string
+
+(** Take a UTF8 string and convert to pdfdocencoding (if no unicode-only
+characters are used) or UTF16BE (if they are)) *)
+val pdfdocstring_of_utf8 : string -> string
+
+(** Build a pdf string in pdfdocencoding (if no unicode-only characters are
+used) or UTF16BE (if they are) *)
+val pdfdocstring_of_codepoints : int list -> string
+
+(** Produce a list of unicode codepoints from a pdfdocencoding or UTF16BE pdf
+document string *)
+val codepoints_of_pdfdocstring : string -> int list
+
+(** {2 Text from strings inside page content} *)
 
 (** The type of text extractors. *)
 type text_extractor
 
 (** Build a text extractor from a document and font object *)
-val text_extractor_of_font : Pdf.pdfdoc -> Pdf.pdfobject -> text_extractor
+val text_extractor_of_font : Pdf.t -> Pdf.pdfobject -> text_extractor
 
 (** Return a list of unicode points from a given extractor and string (for
 example from a [Pdfpages.Op_Tj] or [Op_TJ] operator). *)
 val codepoints_of_text : text_extractor -> string -> int list
 
-(** Same, but return UTF16BE *)
-val utf16be_of_text : text_extractor -> string -> string
+(** Return a list of glyph names from a given extractor and string *)
+val glyphnames_of_text : text_extractor -> string -> string list
 
-(** Same, but return Latin1 (Lossy) *)
-val latin1_string_of_text : text_extractor -> string -> string
+(** {2 Building text for strings inside page content} *)
 
-(** Decode a single character code in a standard font *)
-val decode_char : encoding -> char -> char
-
-(** Decode a single character code in a type3 font to a glyph name *)
-val decode_type3_char : encoding -> char -> string
-
-(** Read the codepoints in a PDF text string, unicode or PDFDocEncoding. *)
-val codepoints_of_textstring : string -> int list
+(** Return the character code for a given unicode codepoint, if it exists in
+this encoding. This is only really suitable for simple stuff like standard 14
+fonts, or editing text in existing fonts. *)
+val charcode_extractor_of_encoding : encoding -> (int -> int option)
 
